package edu.cmu.cs.distributed.intro.ex06;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicInteger;

public class Synchronization {
	private boolean stop = false;
	private volatile boolean stopVol = false;
	
	public void broken1() {
		new Thread(new Runnable() {
			@Override
			public void run() {
				stop = true;
			}
		});
		while(!stop);
	}
	
	public void lessBroken1() {
		new Thread(new Runnable() {
			@Override
			public void run() {
				stopVol = true;
			}
		});
		while(!stopVol);
	}
	
	public void atomicTypes() {
		final AtomicInteger count = new AtomicInteger();
		
		new Thread(new Runnable() {
			@Override
			public void run() {
				for (int i = 0; i < 100; i++) {
					count.incrementAndGet();
				}
			}
		});
		
		for (int i = 0; i < 100; i++) {
			count.incrementAndGet();
		}
		
		while(!count.compareAndSet(200, 0));
	}
	
	public synchronized void syncMethod() {
		// stuff
	}
	
	public void syncBlock() {
		// non-synchronized stuff
		
		synchronized (this) {
			// synchronized on this instance of the Synchronization class
		}
	}

	public void syncCollections() {
		final List<Integer> list = Collections.synchronizedList(new ArrayList<Integer>());
		
		new Thread(new Runnable() {
			@Override
			public void run() {
				for (int i = 0; i < 100; i++) {
					list.add(i);
				}
			}
		});
		
		for (int i = 100; i < 200; i++) {
			list.add(i);
		}
		
		synchronized (list) {
			for (Integer i : list) {
				System.out.println(i);
			}
		}
	}
	
	public void concurrentHashMap() {
		final ConcurrentHashMap<String, Integer> map = new ConcurrentHashMap<String, Integer>();

		Integer old = map.putIfAbsent("Hi", 100);
		if (old == null) {
			// put succeeded
		} else {
			// old is current value
		}
	}
}
